'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
* The FetchInterceptor class
*/
var FetchInterceptor = function () {
  /**
  * Recognize global environment and attach fetch
  */
  function FetchInterceptor() {
    _classCallCheck(this, FetchInterceptor);

    var ENVIRONMENT_IS_REACT_NATIVE = (typeof navigator === 'undefined' ? 'undefined' : _typeof(navigator)) === 'object' && navigator.product === 'ReactNative';
    var ENVIRONMENT_IS_NODE = (typeof process === 'undefined' ? 'undefined' : _typeof(process)) === 'object' && typeof require === 'function';
    var ENVIRONMENT_IS_WEB = (typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object';
    var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';

    if (ENVIRONMENT_IS_REACT_NATIVE) {
      this.env = global;
    } else if (ENVIRONMENT_IS_WORKER) {
      this.env = self;
    } else if (ENVIRONMENT_IS_WEB) {
      this.env = window;
    } else if (ENVIRONMENT_IS_NODE) {
      this.env = global;
    } else {
      throw new Error('Unsupported environment for fetch-intercept');
    }

    this.fetch = this.env.fetch;
  }

  /**
  * Whitelist hooks
  */


  _createClass(FetchInterceptor, [{
    key: 'unregister',


    /**
    * Reset fetch and unregister intercept hooks
    */
    value: function unregister() {
      this.env.fetch = this.fetch;
    }

    /**
    * Hijack global fetch and insert registered hooks if present
    */

  }, {
    key: 'hijack',
    value: function hijack() {
      var self = this;
      this.env.fetch = function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
          for (var _len = arguments.length, request = Array(_len), _key = 0; _key < _len; _key++) {
            request[_key] = arguments[_key];
          }

          var response;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (typeof self.onBeforeRequest === 'function') {
                    self.onBeforeRequest.apply(self, request);
                  }
                  _context.next = 3;
                  return self.fetch.apply(self.env, request);

                case 3:
                  response = _context.sent;

                  if (response.ok) {
                    if (typeof self.onRequestSuccess === 'function') {
                      self.onRequestSuccess(response);
                    }
                  } else {
                    if (typeof self.onRequestFailure === 'function') {
                      self.onRequestFailure(response);
                    }
                  }
                  return _context.abrupt('return', response);

                case 6:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));

        return function () {
          return _ref.apply(this, arguments);
        };
      }();
    }
  }], [{
    key: 'register',


    /**
    * Register intercept hooks & return an interceptor instance
    * @param {object} hooks - The intercept hooks
    * @return {FetchInterceptor} An interceptor object
    */
    value: function register() {
      var hooks = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var interceptor = new this();
      for (var i = 0; i < this.hooks.length; i++) {
        var hook = this.hooks[i];
        if (typeof hooks[hook] === 'function') {
          interceptor[hook] = hooks[hook];
        }
      }
      interceptor.hijack();
      return interceptor;
    }
  }]);

  return FetchInterceptor;
}();

FetchInterceptor.hooks = ['onBeforeRequest', 'onRequestSuccess', 'onRequestFailure'];


module.exports = FetchInterceptor;